# -*- coding: utf-8 -*-

import requests

from requests.packages.urllib3.exceptions import InsecureRequestWarning

COMMON_ERROR_CODES = {
    100: "Unknown error",
    101: "No parameter of API, method or version",
    102: "The requested API does not exist",
    103: "The requested method does not exist",
    104: "The requested version does not support the functionality",
    105: "The logged in session does not have permission",
    106: "Session timeout",
    107: "Session interrupted by duplicate login",
}


class Http(object):
    """HTTP connection to the API.

    This class is responsible for handling all communications with the API.
    """

    def __init__(self, host: str, port: int,
                 username: str, password: str, https: bool = True):
        self.host = host
        self.port = port

        self.username = username
        self.password = password

        self.https = https
        self.verify = True

        self.sid = None

    def disable_https_verify(self):
        """Disable the HTTPS certificate check.
        This should be used only when using self-signed certificate.
        """
        requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
        self.verify = False

    def _get_base_url(self):
        return '{}://{}:{}/webapi'.format(
            'https' if self.https else 'http',
            self.host,
            self.port
        )

    def _login(self):
        params = {
            'format': 'sid',
            'account': self.username,
            'passwd': self.password
        }

        errors = {
            400: "No such account or incorrect password",
            401: "Account disabled",
            402: "Permission denied",
            403: "2-step verification code required",
            404: "Failed to authenticate 2-step verification code",
        }

        response = self.call(
            path='auth.cgi',
            api='SYNO.API.Auth',
            method='Login',
            version=2,
            params=params,
            restricted=False,
            errors=errors,
        )

        self.sid = response['sid']

    def call(self, path: str, api: str, method: str,
             version: int = 1, params: dict = {},
             restricted: bool = True, retried: bool = False,
             errors: dict = {}):
        """Performs an HTTP call to the Synology API."""
        url = '{}/{}'.format(
            self._get_base_url(),
            path,
        )

        if restricted and self.sid is None:
            self._login()

        cookies = {}
        if self.sid is not None:
            cookies['id'] = self.sid

        params['api'] = api
        params['method'] = method
        params['version'] = version

        response = requests.get(
            url,
            verify=self.verify,
            params=params,
            cookies=cookies,
        )

        if response.status_code != 200:
            raise SynologyHttpException(
                "The server answered a wrong status code (code={})".format(
                    response.status_code
                )
            )

        data = response.json()

        if 'success' not in data and (
            'data' not in data or
            'error' not in data
        ):
            raise SynologyHttpException(
                "The output received by the server is malformed"
            )

        if not data['success']:
            code = data['error']['code']

            if code >= 100 and code < 200:
                message = self._get_common_error_message(code)

                # 106 Session timeout
                # 107 Session interrupted by duplicate login
                if code == 106 or code == 107:
                    if not restricted or retried:
                        # We should stop here if:
                        #  1 - Public route, no need to retry the login
                        #  2 - We already retried the route
                        raise SynologyCommonError(
                            code,
                            message,
                        )
                    self._login()
                    # Retry the current request with a new token
                    return self.call(
                        path=path,
                        api=api,
                        method=method,
                        version=version,
                        params=params,
                        restricted=True,
                        retried=True,
                        errors=errors,
                    )

                raise SynologyCommonError(
                    code,
                    message,
                )

            if code in errors:
                message = errors[code]
            else:
                message = "Unknown API error, please check the documentation"

            raise SynologyApiError(
                code,
                message,
            )

        return data['data']

    def _get_common_error_message(self, code):
        """Gets the official message errror from
        the API documentation
        """
        if code not in COMMON_ERROR_CODES:
            return "Unknown common error, please check the documentation"
        return COMMON_ERROR_CODES[code]


class SynologyException(Exception):
    """Base Synology exception."""
    pass


class SynologyHttpException(SynologyException):
    """Synology HTTP exception."""
    pass


class SynologyError(SynologyException):
    """Base Synology error."""
    def __init__(self, code, message):
        self.code = code
        self.message = message
        super(SynologyError, self).__init__(
            "{} (error={})".format(message, code)
        )


class SynologyCommonError(SynologyError):
    """Synology common errror."""
    pass


class SynologyApiError(SynologyError):
    """Synology API error."""
    pass



# -*- coding: utf-8 -*-

from synology_srm.api import Api


class ApiMesh(Api):
    """API Mesh.

    Handles the SYNO.Mesh API namespace.
    """

    def network_wanstatus(self):
        """Gets the network WAN status."""
        return self.http.call(
            path='entry.cgi',
            api='SYNO.Mesh.Network.WANStatus',
            method='get',
            version=1,
        )

    def network_wifidevice(self):
        """Gets the network Wi-Fi devices."""
        response = self.http.call(
            path='entry.cgi',
            api='SYNO.Mesh.Network.WifiDevice',
            method='get',
            version=1,
        )

        return response['devices']

    def system_info(self):
        """Gets the SRM system info."""
        return self.http.call(
            path='entry.cgi',
            api='SYNO.Mesh.System.Info',
            method='get',
            version=1,
        )



# -*- coding: utf-8 -*-

from synology_srm.api import Api

INTERVAL_VALUES = ['live', 'day', 'week', 'month']

class ApiCore(Api):
    """API Core.

    Handles the SYNO.Core API namespace.
    """

    def system_utilization(self):
        """Gets the system utilization."""
        return self.http.call(
            path='entry.cgi',
            api='SYNO.Core.System.Utilization',
            method='get',
            version=1,
        )

    def ddns_extip(self):
        """Gets the external IP address."""
        return self.http.call(
            path='entry.cgi',
            api='SYNO.Core.DDNS.ExtIP',
            method='list',
            version=1,
        )

    def ddns_record(self):
        """Gets the DDNS record."""
        return self.http.call(
            path='entry.cgi',
            api='SYNO.Core.DDNS.Record',
            method='list',
            version=1,
        )

    def network_nsm_device(self, filters={}):
        """Gets the network NSM device."""
        response = self.http.call(
            path='entry.cgi',
            api='SYNO.Core.Network.NSM.Device',
            method='get',
            version=1,
        )

        return self._filter(response['devices'], filters)

    def ngfw_traffic(self, interval):
        """Gets network traffic statistics for the specified interval."""
        if interval not in INTERVAL_VALUES:
            raise AttributeError('Interval unknown, must be one of {}'.format(INTERVAL_VALUES))

        params = {
            'mode': 'net_l7',
            'interval': interval
        }

        return self.http.call(
            path='entry.cgi',
            api='SYNO.Core.NGFW.Traffic',
            method='get',
            params=params,
            version=1,
        )



# -*- coding: utf-8 -*-

from synology_srm.http import Http


class Api(object):
    """API.

    Base class for all API namespaces.
    """

    def __init__(self, http: Http):
        self.http = http

    def _filter(self, elements, filters):
        """Filter elements with a list of constraints
        Each filter needs to be evaluate to true to return the element
        """
        if not isinstance(filters, dict) or not filters:
            return elements

        def _filter_element(element, filters):
            for key, value in filters.items():
                if element[key] != value:
                    return False
            return True

        return list(filter(lambda e: _filter_element(e, filters), elements))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import io
import os
import sys

from shutil import rmtree
from setuptools import find_packages, setup, Command

NAME = 'synology-srm'
DESCRIPTION = 'Synology SRM API'

REQUIRED = [
    'requests',
]

here = os.path.abspath(os.path.dirname(__file__))

try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

about = {}
with open(os.path.join(here, 'synology_srm', '__version__.py')) as f:
    exec(f.read(), about)

setup(
    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Julien Brochet',
    author_email='julien@aerialls.io',
    python_requires='>=3.6',
    url='https://github.com/aerialls/synology-srm',
    packages=find_packages(exclude=('tests',)),
    install_requires=REQUIRED,
    include_package_data=True,
    license='MIT',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ]
)
